@@ -1,14 +1,9 @@
 # -*- coding: utf-8 -*-
 #
-# This file is part of cclib (http://cclib.github.io), a library for parsing
-# and interpreting the results of computational chemistry packages.
+# Copyright (c) 2017, the cclib development team
 #
-# Copyright (C) 2006-2014, the cclib development team
-#
-# The library is free software, distributed under the terms of
-# the GNU Lesser General Public version 2.1 or later. You should have
-# received a copy of the license along with cclib. You can also access
-# the full license online at http://www.gnu.org/copyleft/lgpl.html.
+# This file is part of cclib (http://cclib.github.io) and is distributed under
+# the terms of the BSD 3-Clause License.
 
 """C-squared population analysis."""
 
@@ -16,32 +11,32 @@ import random
 
 import numpy
 
-from .population import Population
+from cclib.method.population import Population
 
 
 class CSPA(Population):
     """The C-squared population analysis."""
-    
+
     def __init__(self, *args):
 
         # Call the __init__ method of the superclass.
         super(CSPA, self).__init__(logname="CSPA", *args)
-        
+
     def __str__(self):
         """Return a string representation of the object."""
-        return "CSPA of" % (self.data)
+        return "CSPA of %s" % (self.data)
 
     def __repr__(self):
         """Return a representation of the object."""
         return 'CSPA("%s")' % (self.data)
-    
+
     def calculate(self, indices=None, fupdate=0.05):
         """Perform the C squared population analysis.
-        
+
         Inputs:
            indices - list of lists containing atomic orbital indices of fragments
         """
-    
+
         # Do we have the needed info in the parser?
         if not hasattr(self.data, "mocoeffs"):
             self.logger.error("Missing mocoeffs")
@@ -99,20 +94,25 @@ class CSPA(Population):
         self.logger.info("Creating fragcharges: array[1]")
         size = len(self.fragresults[0][0])
         self.fragcharges = numpy.zeros([size], "d")
-        
+        alpha = numpy.zeros([size], "d")
+        if unrestricted:
+            beta = numpy.zeros([size], "d")
+
         for spin in range(len(self.fragresults)):
 
             for i in range(self.data.homos[spin] + 1):
 
                 temp = numpy.reshape(self.fragresults[spin][i], (size,))
                 self.fragcharges = numpy.add(self.fragcharges, temp)
-        
+                if spin == 0:
+                    alpha = numpy.add(alpha, temp)
+                elif spin == 1:
+                    beta = numpy.add(beta, temp)
+
         if not unrestricted:
             self.fragcharges = numpy.multiply(self.fragcharges, 2)
+        else:
+            self.logger.info("Creating fragspins: array[1]")
+            self.fragspins = numpy.subtract(alpha, beta)
 
         return True
-
-
-if __name__ == "__main__":
-    import doctest, cspa
-    doctest.testmod(cspa, verbose=False)
