@@ -1,14 +1,9 @@
 # -*- coding: utf-8 -*-
 #
-# This file is part of cclib (http://cclib.github.io), a library for parsing
-# and interpreting the results of computational chemistry packages.
+# Copyright (c) 2017, the cclib development team
 #
-# Copyright (C) 2006-2014, the cclib development team
-#
-# The library is free software, distributed under the terms of
-# the GNU Lesser General Public version 2.1 or later. You should have
-# received a copy of the license along with cclib. You can also access
-# the full license online at http://www.gnu.org/copyleft/lgpl.html.
+# This file is part of cclib (http://cclib.github.io) and is distributed under
+# the terms of the BSD 3-Clause License.
 
 """Parser for Jaguar output files"""
 
@@ -17,8 +12,8 @@ import re
 
 import numpy
 
-from . import logfileparser
-from . import utils
+from cclib.parser import logfileparser
+from cclib.parser import utils
 
 
 class Jaguar(logfileparser.Logfile):
@@ -28,7 +23,7 @@ class Jaguar(logfileparser.Logfile):
 
         # Call the __init__ method of the superclass
         super(Jaguar, self).__init__(logname="Jaguar", *args, **kwargs)
-        
+
     def __str__(self):
         """Return a string representation of the object."""
         return "Jaguar output file %s" % (self.filename)
@@ -44,12 +39,6 @@ class Jaguar(logfileparser.Logfile):
         (1) To handle orbitals of E symmetry, retain everything before the /
         (2) Replace two p's by "
         (2) Replace any remaining single p's by '
-
-        >>> t = Jaguar("dummyfile").normalisesym
-        >>> labels = ['A', 'A1', 'Ag', 'Ap', 'App', "A1p", "A1pp", "E1pp/Ap"]
-        >>> answers = map(t, labels)
-        >>> print answers
-        ['A', 'A1', 'Ag', "A'", 'A"', "A1'", 'A1"', 'E1"']
         """
         ans = label.split("/")[0].replace("pp", '"').replace("p", "'")
         return ans
@@ -71,6 +60,14 @@ class Jaguar(logfileparser.Logfile):
     def extract(self, inputfile, line):
         """Extract information from the file object inputfile."""
 
+        # Extract the version number first
+        if "Jaguar version" in line:
+            self.metadata["package_version"] = line.split()[3][:-1]
+
+        # Extract the basis set name
+        if line[2:12] == "basis set:":
+            self.metadata["basis_set"] = line.split()[2]
+
         # Extract charge and multiplicity
         if line[2:22] == "net molecular charge":
             self.set_attribute('charge', int(line.split()[-1]))
@@ -83,7 +80,7 @@ class Jaguar(logfileparser.Logfile):
         # the atomic orbitals correctly, so use it to build atombasis.
         #
         #  Gaussian basis set information
-        #  
+        #
         #                                                        renorm    mfac*renorm
         #   atom    fn   prim  L        z            coef         coef         coef
         # -------- ----- ---- --- -------------  -----------  -----------  -----------
@@ -161,8 +158,8 @@ class Jaguar(logfileparser.Logfile):
             # This length of atombasis should always be the number of atoms.
             self.set_attribute('natom', len(self.atombasis))
 
-        #  Effective Core Potential 
-        #  
+        #  Effective Core Potential
+        #
         #  Atom      Electrons represented by ECP
         # Mo                    36
         #              Maximum angular term         3
@@ -196,14 +193,13 @@ class Jaguar(logfileparser.Logfile):
                 if len(line.split()) == 2:
                     self.coreelectrons.append(int(line.split()[1]))
                 line = next(inputfile)
-                
 
         if line[2:14] == "new geometry" or line[1:21] == "Symmetrized geometry" or line.find("Input geometry") > 0:
         # Get the atom coordinates
             if not hasattr(self, "atomcoords") or line[1:21] == "Symmetrized geometry":
                 # Wipe the "Input geometry" if "Symmetrized geometry" present
                 self.atomcoords = []
-            p = re.compile("(\D+)\d+") # One/more letters followed by a number
+            p = re.compile("(\D+)\d+")  # One/more letters followed by a number
             atomcoords = []
             atomnos = []
             angstrom = next(inputfile)
@@ -221,6 +217,7 @@ class Jaguar(logfileparser.Logfile):
 
         # Hartree-Fock energy after SCF
         if line[1:18] == "SCFE: SCF energy:":
+            self.metadata["methods"].append("HF")
             if not hasattr(self, "scfenergies"):
                 self.scfenergies = []
             temp = line.strip().split()
@@ -230,6 +227,7 @@ class Jaguar(logfileparser.Logfile):
 
         # Energy after LMP2 correction
         if line[1:18] == "Total LMP2 Energy":
+            self.metadata["methods"].append("LMP2")
             if not hasattr(self, "mpenergies"):
                 self.mpenergies = [[]]
             lmp2energy = float(line.split()[-1])
@@ -287,30 +285,34 @@ class Jaguar(logfileparser.Logfile):
         # etot   1  N  N  0  N    -382.08751881733           2.3E-03  1.4E-01
         # etot   2  Y  Y  0  N    -382.27486018708  1.9E-01  1.4E-03  5.7E-02
                 temp = line.split()[7:]
-                if len(temp)==3:
+                if len(temp) == 3:
                     denergy = float(temp[0])
                 else:
-                    denergy = 0 # Should really be greater than target value
-                                # or should we just ignore the values in this line
+                    denergy = 0  # Should really be greater than target value
+                                 # or should we just ignore the values in this line
                 ddensity = float(temp[-2])
                 maxdiiserr = float(temp[-1])
                 if not self.geoopt:
                     values.append([denergy, ddensity])
                 else:
                     values.append([ddensity])
-                line = next(inputfile)
+                try:
+                    line = next(inputfile)
+                except StopIteration:
+                    self.logger.warning('File terminated before end of last SCF! Last error: {}'.format(maxdiiserr))
+                    break
             self.scfvalues.append(values)
 
         # MO energies and symmetries.
         # Jaguar 7.0: provides energies and symmetries for both
         #   restricted and unrestricted calculations, like this:
-        #     Alpha Orbital energies/symmetry label: 
-        #     -10.25358 Bu  -10.25353 Ag  -10.21931 Bu  -10.21927 Ag     
-        #     -10.21792 Bu  -10.21782 Ag  -10.21773 Bu  -10.21772 Ag     
+        #     Alpha Orbital energies/symmetry label:
+        #     -10.25358 Bu  -10.25353 Ag  -10.21931 Bu  -10.21927 Ag
+        #     -10.21792 Bu  -10.21782 Ag  -10.21773 Bu  -10.21772 Ag
         #     ...
         # Jaguar 6.5: prints both only for restricted calculations,
         #   so for unrestricted calculations the output it looks like this:
-        #     Alpha Orbital energies: 
+        #     Alpha Orbital energies:
         #     -10.25358  -10.25353  -10.21931  -10.21927  -10.21792  -10.21782
         #     -10.21773  -10.21772  -10.21537  -10.21537   -1.02078   -0.96193
         #     ...
@@ -328,7 +330,7 @@ class Jaguar(logfileparser.Logfile):
                 self.moenergies = []
             if issyms and not hasattr(self, "mosyms"):
                     self.mosyms = []
-            
+
             # Grow moeneriges/mosyms and make sure they are empty when
             #   parsed multiple times - currently cclib returns only
             #   the final output (ex. in a geomtry optimization).
@@ -339,7 +341,7 @@ class Jaguar(logfileparser.Logfile):
                 if len(self.mosyms) < spin+1:
                     self.mosyms.append([])
                 self.mosyms[spin] = []
-            
+
             line = next(inputfile).split()
             while len(line) > 0:
                 if issyms:
@@ -353,7 +355,7 @@ class Jaguar(logfileparser.Logfile):
                     syms = [self.normalisesym(s) for s in syms]
                     self.mosyms[spin].extend(syms)
                 line = next(inputfile).split()
-            
+
             line = next(inputfile)
 
         # The second trigger string is in the version 8.3 unit test and the first one was
@@ -361,13 +363,13 @@ class Jaguar(logfileparser.Logfile):
         # the line with occupations is missing in each block. Here is a fragment of this block
         # from version 8.3:
         #
-        # ***************************************** 
-        # 
+        # *****************************************
+        #
         # occupied + virtual orbitals: final wave function
-        # 
-        # ***************************************** 
-        #   
-        #   
+        #
+        # *****************************************
+        #
+        #
         #                              1         2         3         4         5
         #  eigenvalues-            -11.04064 -11.04058 -11.03196 -11.03196 -11.02881
         #  occupations-              2.00000   2.00000   2.00000   2.00000   2.00000
@@ -379,13 +381,13 @@ class Jaguar(logfileparser.Logfile):
            line.find("occupied + virtual orbitals: final wave function") > 0:
 
             self.skip_lines(inputfile, ['b', 's', 'b', 'b'])
-            
-            if not hasattr(self,"mocoeffs"):
+
+            if not hasattr(self, "mocoeffs"):
                 self.mocoeffs = []
 
             aonames = []
             lastatom = "X"
-            
+
             readatombasis = False
             if not hasattr(self, "atombasis"):
                 self.atombasis = []
@@ -399,7 +401,7 @@ class Jaguar(logfileparser.Logfile):
             for s in range(spin):
                 mocoeffs = numpy.zeros((len(self.moenergies[s]), self.nbasis), "d")
 
-                if s == 1: #beta case
+                if s == 1:  # beta case
                     self.skip_lines(inputfile, ['s', 'b', 'title', 'b', 's', 'b', 'b'])
 
                 for k in range(0, len(self.moenergies[s]), 5):
@@ -416,7 +418,7 @@ class Jaguar(logfileparser.Logfile):
                     for i in range(self.nbasis):
 
                         info = line.split()
-                        
+
                         # Fill atombasis only first time around.
                         if readatombasis and k == 0:
                             orbno = int(info[0])
@@ -427,24 +429,24 @@ class Jaguar(logfileparser.Logfile):
                                 atomno = int(atom[1:])
                             self.atombasis[atomno-1].append(orbno-1)
 
-                        if not hasattr(self,"aonames"):
+                        if not hasattr(self, "aonames"):
                             if lastatom != info[1]:
                                 scount = 1
                                 pcount = 3
-                                dcount = 6 #six d orbitals in Jaguar
+                                dcount = 6  # six d orbitals in Jaguar
 
                             if info[2] == 'S':
-                                aonames.append("%s_%i%s"%(info[1], scount, info[2]))
+                                aonames.append("%s_%i%s" % (info[1], scount, info[2]))
                                 scount += 1
-                        
+
                             if info[2] == 'X' or info[2] == 'Y' or info[2] == 'Z':
-                                aonames.append("%s_%iP%s"%(info[1], pcount / 3, info[2]))
+                                aonames.append("%s_%iP%s" % (info[1], pcount / 3, info[2]))
                                 pcount += 1
-                        
+
                             if info[2] == 'XX' or info[2] == 'YY' or info[2] == 'ZZ' or \
                                info[2] == 'XY' or info[2] == 'XZ' or info[2] == 'YZ':
 
-                                aonames.append("%s_%iD%s"%(info[1], dcount / 6, info[2]))
+                                aonames.append("%s_%iD%s" % (info[1], dcount / 6, info[2]))
                                 dcount += 1
 
                             lastatom = info[1]
@@ -454,18 +456,18 @@ class Jaguar(logfileparser.Logfile):
 
                         line = next(inputfile)
 
-                    if not hasattr(self,"aonames"):
+                    if not hasattr(self, "aonames"):
                         self.aonames = aonames
 
                     offset += 5
                 self.mocoeffs.append(mocoeffs)
 
-        #  Atomic charges from Mulliken population analysis: 
-        #   
-        # Atom       C1           C2           C3           C4           C5      
+        #  Atomic charges from Mulliken population analysis:
+        #
+        # Atom       C1           C2           C3           C4           C5
         # Charge    0.00177     -0.06075     -0.05956      0.00177     -0.06075
-        #   
-        # Atom       H6           H7           H8           C9           C10  
+        #
+        # Atom       H6           H7           H8           C9           C10
         # ...
         if line.strip() == "Atomic charges from Mulliken population analysis:":
 
@@ -502,7 +504,7 @@ class Jaguar(logfileparser.Logfile):
                     temp = list(map(float, next(inputfile).split()[1:]))
                     self.aooverlaps[j, i:(i+len(temp))] = temp
                     self.aooverlaps[i:(i+len(temp)), j] = temp
-            
+
         if line[2:24] == "start of program geopt":
             if not self.geoopt:
                 # Need to keep only the RMS density change info
@@ -520,13 +522,13 @@ class Jaguar(logfileparser.Logfile):
         #  energy:            -382.30219111487 hartrees
         #  [ turning on trust-radius adjustment ]
         #  ** restarting optimization from step    6 **
-        #  
-        #  
+        #
+        #
         #  Level shifts adjusted to satisfy step-size constraints
         #   Step size:    0.0360704
         #   Cos(theta):   0.8789215
         #   Final level shift:  -8.6176299E-02
-        #  
+        #
         #  energy change:           2.5819E-04 .  (  5.0000E-05 )
         #  gradient maximum:        5.0947E-03 .  (  4.5000E-04 )
         #  gradient rms:            1.2996E-03 .  (  3.0000E-04 )
@@ -570,7 +572,7 @@ class Jaguar(logfileparser.Logfile):
             # ther is also no energy change when the optimization is restarted
             # from step 1 (since step 1 had no change).
             values = []
-            target_index = 0                
+            target_index = 0
             if (gopt_step == 1) or restarting_from_1:
                 values.append(0.0)
                 target_index = 1
@@ -585,7 +587,7 @@ class Jaguar(logfileparser.Logfile):
 
         # IR output looks like this:
         #   frequencies        72.45   113.25   176.88   183.76   267.60   312.06
-        #   symmetries       Au       Bg       Au       Bu       Ag       Bg      
+        #   symmetries       Au       Bg       Au       Bu       Ag       Bg
         #   intensities         0.07     0.00     0.28     0.52     0.00     0.00
         #   reduc. mass         1.90     0.74     1.06     1.42     1.19     0.85
         #   force const         0.01     0.01     0.02     0.03     0.05     0.05
@@ -615,7 +617,7 @@ class Jaguar(logfileparser.Logfile):
                 if "intensities" in line:
                     intensities = True
                 line = next(inputfile)
-            
+
             # In older version, the last block had an extra blank line after it,
             # which could be caught. This is not true in newer version (including 8.3),
             # but in general it would be better to bound this loop more strictly.
@@ -628,27 +630,27 @@ class Jaguar(logfileparser.Logfile):
                 # Append the frequencies.
                 self.vibfreqs.extend(list(map(float, freqs.split()[1:])))
                 line = next(inputfile).split()
-                
+
                 # May skip symmetries (older Jaguar versions).
                 if line[0] == "symmetries":
                     if not hasattr(self, "vibsyms"):
                         self.vibsyms = []
                     self.vibsyms.extend(list(map(self.normalisesym, line[1:])))
-                    line = next(inputfile).split()                                
+                    line = next(inputfile).split()
                 if intensities:
                     if not hasattr(self, "vibirs"):
                         self.vibirs = []
                     self.vibirs.extend(list(map(float, line[1:])))
-                    line = next(inputfile).split()                                
+                    line = next(inputfile).split()
                 if forceconstants:
                     line = next(inputfile)
 
                 # Start parsing the displacements.
                 # Variable 'q' holds up to 7 lists of triplets.
-                q = [ [] for i in range(7) ]
+                q = [[] for i in range(7)]
                 for n in range(self.natom):
                     # Variable 'p' holds up to 7 triplets.
-                    p = [ [] for i in range(7) ]
+                    p = [[] for i in range(7)]
                     for i in range(3):
                         line = next(inputfile)
                         disps = [float(disp) for disp in line.split()[2:]]
@@ -667,7 +669,7 @@ class Jaguar(logfileparser.Logfile):
             self.vibdisps = numpy.array(self.vibdisps, "d")
             if hasattr(self, "vibirs"):
                 self.vibirs = numpy.array(self.vibirs, "d")
-                
+
         # Parse excited state output (for CIS calculations).
         # Jaguar calculates only singlet states.
         if line[2:15] == "Excited State":
@@ -699,8 +701,3 @@ class Jaguar(logfileparser.Logfile):
                 line = next(inputfile)
             strength = float(line.split()[-1])
             self.etoscs.append(strength)
-
-        
-if __name__ == "__main__":
-    import doctest, jaguarparser
-    doctest.testmod(jaguarparser, verbose=False)
