@@ -1,15 +1,9 @@
 # -*- coding: utf-8 -*-
 #
-# This file is part of cclib (http://cclib.github.io), a library for parsing
-# and interpreting the results of computational chemistry packages.
+# Copyright (c) 2017, the cclib development team
 #
-# Copyright (C) 2006-2014, the cclib development team
-#
-# The library is free software, distributed under the terms of
-# the GNU Lesser General Public version 2.1 or later. You should have
-# received a copy of the license along with cclib. You can also access
-# the full license online at http://www.gnu.org/copyleft/lgpl.html.
-
+# This file is part of cclib (http://cclib.github.io) and is distributed under
+# the terms of the BSD 3-Clause License.
 """Generic output file parser and related tools"""
 
 
@@ -26,39 +20,79 @@ import zipfile
 
 import numpy
 
-from . import utils
-from .data import ccData
+from cclib.parser import utils
+from cclib.parser.data import ccData
+from cclib.parser.data import ccData_optdone_bool
 
 
 # This seems to avoid a problem with Avogadro.
-logging.logMultiprocessing =  0
+logging.logMultiprocessing = 0
 
 
 class myBZ2File(bz2.BZ2File):
     """Return string instead of bytes"""
     def __next__(self):
-        line = super().__next__()
+        line = super(bz2.BZ2File, self).__next__()
         return line.decode("ascii", "replace")
 
+    def next(self):
+        line = self.__next__()
+        return line
+
+
 class myGzipFile(gzip.GzipFile):
     """Return string instead of bytes"""
     def __next__(self):
-        line = super().__next__()
+        super_ob = super(gzip.GzipFile, self)
+        # seemingly different versions of gzip can have either next or __next__
+        if hasattr(super_ob, 'next'):
+            line = super_ob.next()
+        else:
+            line = super_ob.__next__()
         return line.decode("ascii", "replace")
 
+    def next(self):
+        line = self.__next__()
+        return line
+
+
+class myFileinputFile(fileinput.FileInput):
+    """Implement next() method"""
+    def next(self):
+        line = next(self)
+        return line
+
 
 class FileWrapper(object):
-    """Wrap a file object so that we can maintain position"""
+    """Wrap a file-like object or stream with some custom tweaks"""
 
-    def __init__(self, file):
-        self.file = file
-        self.pos = 0
-        self.file.seek(0, 2)
-        self.size = self.file.tell()
-        self.file.seek(0, 0)
+    def __init__(self, source, pos=0):
+
+        self.src = source
+
+        # Most file-like objects have seek and tell methods, but streams returned
+        # by urllib.urlopen in Python2 do not, which will raise an AttributeError
+        # in this code. On the other hand, in Python3 these methods do exist since
+        # urllib uses the stream class in the io library, but they raise a different
+        # error, namely io.UnsupportedOperation. That is why it is hard to be more
+        # specific with except block here.
+        try:
+            self.src.seek(0, 2)
+            self.size = self.src.tell()
+            self.src.seek(pos, 0)
+
+        except (AttributeError, IOError, io.UnsupportedOperation):
+            # Stream returned by urllib should have size information.
+            if hasattr(self.src, 'headers') and 'content-length' in self.src.headers:
+                self.size = int(self.src.headers['content-length'])
+            else:
+                self.size = pos
+
+        # Assume the position is what was passed to the constructor.
+        self.pos = pos
 
     def next(self):
-        line = next(self.file)
+        line = next(self.src)
         self.pos += len(line)
         return line
 
@@ -69,20 +103,35 @@ class FileWrapper(object):
         return self
 
     def close(self):
-        self.file.close()
+        self.src.close()
 
     def seek(self, pos, ref):
-        self.file.seek(pos, ref)
+
+        # If we are seeking to end, we can emulate it usually. As explained above,
+        # we cannot be too specific with the except clause due to differences
+        # between Python2 and 3. Yet another reason to drop Python 2 soon!
+        try:
+            self.src.seek(pos, ref)
+        except:
+            if ref == 2:
+                self.src.read()
+            else:
+                raise
+
+        if ref == 0:
+            self.pos = pos
+        if ref == 1:
+            self.pos += pos
+        if ref == 2 and hasattr(self, 'size'):
+            self.pos = self.size
 
 
-def openlogfile(filename):
-    """Return a file object given a filename.
+def openlogfile(filename, object=None):
+    """Return a file object given a filename or if object specified decompresses it
+    if needed and wrap it up.
 
-    Given the filename of a log file or a gzipped, zipped, or bzipped
-    log file, this function returns a regular Python file object.
-    
-    Given an address starting with http://, this function retrieves the url
-    and returns a file object using a temporary file.
+    Given the filename or file object of a log file or a gzipped, zipped, or bzipped
+    log file, this function returns a file-like object.
 
     Given a list of filenames, this function returns a FileInput object,
     which can be used for seamless iteration without concatenation.
@@ -92,33 +141,39 @@ def openlogfile(filename):
     if type(filename) in [str, str]:
 
         extension = os.path.splitext(filename)[1]
-        
+
         if extension == ".gz":
-            fileobject = myGzipFile(filename, "r")
+            fileobject = myGzipFile(filename, "r", fileobj=object)
 
         elif extension == ".zip":
-            zip = zipfile.ZipFile(filename, "r")
+            zip = zipfile.ZipFile(object, "r") if object else zipfile.ZipFile(filename, "r")
             assert len(zip.namelist()) == 1, "ERROR: Zip file contains more than 1 file"
             fileobject = io.StringIO(zip.read(zip.namelist()[0]).decode("ascii", "ignore"))
 
         elif extension in ['.bz', '.bz2']:
             # Module 'bz2' is not always importable.
-            assert bz2 != None, "ERROR: module bz2 cannot be imported"
-            fileobject = myBZ2File(filename, "r")
+            assert bz2 is not None, "ERROR: module bz2 cannot be imported"
+            fileobject = myBZ2File(object, "r") if object else myBZ2File(filename, "r")
 
         else:
-            fileobject = FileWrapper(io.open(filename, "r", errors='ignore'))
+            # Assuming that object is text file encoded in utf-8
+            fileobject = io.StringIO(object.decode('utf-8')) if object \
+                    else FileWrapper(io.open(filename, "r", errors='ignore'))
 
         return fileobject
-    
+
     elif hasattr(filename, "__iter__"):
-    
+
+        # This is needed, because fileinput will assume stdin when filename is empty.
+        if len(filename) == 0:
+            return None
+
         # Compression (gzip and bzip) is supported as of Python 2.5.
         if sys.version_info[0] >= 2 and sys.version_info[1] >= 5:
             fileobject = fileinput.input(filename, openhook=fileinput.hook_compressed)
         else:
-            fileobject = fileinput.input(filename)
-        
+            fileobject = myFileinputFile(filename)
+
         return fileobject
 
 
@@ -126,24 +181,27 @@ class Logfile(object):
     """Abstract class for logfile objects.
 
     Subclasses defined by cclib:
-        ADF, GAMESS, GAMESSUK, Gaussian, Jaguar, Molpro, NWChem, ORCA,
-          Psi, QChem
-    
+        ADF, DALTON, GAMESS, GAMESSUK, Gaussian, Jaguar, Molpro, MOPAC,
+        NWChem, ORCA, Psi, Q-Chem
     """
 
     def __init__(self, source, loglevel=logging.INFO, logname="Log",
-                    logstream=sys.stdout, datatype=ccData, **kwds):
+                 logstream=sys.stdout, datatype=ccData_optdone_bool, **kwds):
         """Initialise the Logfile object.
 
         This should be called by a subclass in its own __init__ method.
 
         Inputs:
-            source - a single logfile, a list of logfiles, or input stream
+            source - a logfile, list of logfiles, or stream with at least a read method
+            loglevel - integer corresponding to a log level from the logging module
+            logname - name of the source logfile passed to this constructor
+            logstream - where to output the logging information
+            datatype - class to use for gathering data attributes
         """
 
-        # Set the filename to source if it is a string or a list of filenames.
-        # In the case of an input stream, set some arbitrary name and the stream.
-        # Elsewise, raise an Exception.
+        # Set the filename to source if it is a string or a list of strings, which are
+        # assumed to be filenames. Otherwise, assume the source is a file-like object
+        # if it has a read method, and we will try to use it like a stream.
         if isinstance(source, str):
             self.filename = source
             self.isstream = False
@@ -162,7 +220,7 @@ class Logfile(object):
         # Presently in cclib, all parser instances of the same class use the same logger,
         #   which means that care needs to be taken not to duplicate handlers.
         self.loglevel = loglevel
-        self.logname  = logname
+        self.logname = logname
         self.logger = logging.getLogger('%s %s' % (self.logname, self.filename))
         self.logger.setLevel(self.loglevel)
         if len(self.logger.handlers) == 0:
@@ -170,6 +228,13 @@ class Logfile(object):
             handler.setFormatter(logging.Formatter("[%(name)s %(levelname)s] %(message)s"))
             self.logger.addHandler(handler)
 
+        # Set up the metadata.
+        if not hasattr(self, "metadata"):
+            self.metadata = {}
+            self.metadata["package"] = self.logname
+            self.metadata["methods"] = []
+
+
         # Periodic table of elements.
         self.table = utils.PeriodicTable()
 
@@ -181,21 +246,20 @@ class Logfile(object):
         # is used, which might have more consequences in the future.
         optdone_as_list = kwds.get("optdone_as_list", False) or kwds.get("future", False)
         optdone_as_list = optdone_as_list if isinstance(optdone_as_list, bool) else False
-        if not optdone_as_list:
-            from .data import ccData_optdone_bool
-            self.datatype = ccData_optdone_bool
+        if optdone_as_list:
+            self.datatype = ccData
 
     def __setattr__(self, name, value):
 
         # Send info to logger if the attribute is in the list self._attrlist.
         if name in getattr(self, "_attrlist", {}) and hasattr(self, "logger"):
-                    
+
             # Call logger.info() only if the attribute is new.
             if not hasattr(self, name):
                 if type(value) in [numpy.ndarray, list]:
-                    self.logger.info("Creating attribute %s[]" %name)
+                    self.logger.info("Creating attribute %s[]" % name)
                 else:
-                    self.logger.info("Creating attribute %s: %s" %(name, str(value)))
+                    self.logger.info("Creating attribute %s: %s" % (name, str(value)))
 
         # Set the attribute.
         object.__setattr__(self, name, value)
@@ -206,11 +270,11 @@ class Logfile(object):
         # Check that the sub-class has an extract attribute,
         #  that is callable with the proper number of arguemnts.
         if not hasattr(self, "extract"):
-            raise AttributeError("Class %s has no extract() method." %self.__class__.__name__)
+            raise AttributeError("Class %s has no extract() method." % self.__class__.__name__)
         if not callable(self.extract):
-            raise AttributeError("Method %s._extract not callable." %self.__class__.__name__)
+            raise AttributeError("Method %s._extract not callable." % self.__class__.__name__)
         if len(inspect.getargspec(self.extract)[0]) != 3:
-            raise AttributeError("Method %s._extract takes wrong number of arguments." %self.__class__.__name__)
+            raise AttributeError("Method %s._extract takes wrong number of arguments." % self.__class__.__name__)
 
         # Save the current list of attributes to keep after parsing.
         # The dict of self should be the same after parsing.
@@ -221,7 +285,7 @@ class Logfile(object):
         if not self.isstream:
             inputfile = openlogfile(self.filename)
         else:
-            inputfile = self.stream
+            inputfile = FileWrapper(self.stream)
 
         # Intialize self.progress
         is_compressed = isinstance(inputfile, myGzipFile) or isinstance(inputfile, myBZ2File)
@@ -238,7 +302,6 @@ class Logfile(object):
         # Loop over lines in the file object and call extract().
         # This is where the actual parsing is done.
         for line in inputfile:
-
             self.updateprogress(inputfile, "Unsupported information", cupdate)
 
             # This call should check if the line begins a section of extracted data.
@@ -263,9 +326,12 @@ class Logfile(object):
         if not hasattr(self, "nmo") and hasattr(self, "nbasis"):
             self.nmo = self.nbasis
 
-        # Creating deafult coreelectrons array.
+        # Create a default coreelectrons array, unless it's impossible
+        # to determine.
         if not hasattr(self, "coreelectrons") and hasattr(self, "natom"):
             self.coreelectrons = numpy.zeros(self.natom, "i")
+        if hasattr(self, "incorrect_coreelectrons"):
+            self.__delattr__("coreelectrons")
 
         # Create the data object we want to return. This is normally ccData, but can be changed
         # by passing the datatype argument to the constructor. All supported cclib attributes
@@ -274,7 +340,7 @@ class Logfile(object):
         # There is the possibility of passing assitional argument via self.data_args, but
         # we use this sparingly in cases where we want to limit the API with options, etc.
         data = self.datatype(attributes=self.__dict__)
-                
+
         # Now make sure that the cclib attributes in the data object are all the correct type,
         # including arrays and lists of arrays.
         data.arrayify()
@@ -315,7 +381,7 @@ class Logfile(object):
         contain appropriate doctests. If is not overwritten, this is detected
         as an error by unit tests.
         """
-        return "ERROR: This should be overwritten by this subclass"
+        raise NotImplementedError("normalisesym(self, symlabel) must be overriden by the parser.")
 
     def float(self, number):
         """Convert a string to a float.
@@ -324,20 +390,12 @@ class Logfile(object):
         including avoiding the problem with Ds instead of Es in scientific notation.
         Another point is converting string signifying numerical problems (*****)
         to something we can manage (Numpy's NaN).
-
-        >>> t = Logfile("dummyfile")
-        >>> t.float("123.2323E+02")
-        12323.23
-        >>> t.float("123.2323D+02")
-        12323.23
-        >>> t.float("*****")
-        nan
         """
 
         if list(set(number)) == ['*']:
             return numpy.nan
 
-        return float(number.replace("D","E"))
+        return float(number.replace("D", "E"))
 
     def set_attribute(self, name, value, check=True):
         """Set an attribute and perform a check when it already exists.
@@ -345,7 +403,7 @@ class Logfile(object):
         Note that this can be used for scalars and lists alike, whenever we want
         to set a value for an attribute. By default we want to check that
         the value does not change if the attribute already exists, and this function
-        is a good place to add more tests in the future.        
+        is a good place to add more tests in the future.
         """
         if check and hasattr(self, name):
             try:
@@ -367,9 +425,9 @@ class Logfile(object):
         """
 
         expected_characters = {
-            '-' : ['dashes', 'd'],
-            '=' : ['equals', 'e'],
-            '*' : ['stars', 's'],
+            '-': ['dashes', 'd'],
+            '=': ['equals', 'e'],
+            '*': ['stars', 's'],
         }
 
         lines = []
@@ -406,8 +464,3 @@ class Logfile(object):
         return lines
 
     skip_line = lambda self, inputfile, expected: self.skip_lines(inputfile, [expected])
-
-
-if __name__ == "__main__":
-    import doctest
-    doctest.testmod()
