@@ -1,14 +1,9 @@
 # -*- coding: utf-8 -*-
 #
-# This file is part of cclib (http://cclib.github.io), a library for parsing
-# and interpreting the results of computational chemistry packages.
+# Copyright (c) 2017, the cclib development team
 #
-# Copyright (C) 2008-2014, the cclib development team
-#
-# The library is free software, distributed under the terms of
-# the GNU Lesser General Public version 2.1 or later. You should have
-# received a copy of the license along with cclib. You can also access
-# the full license online at http://www.gnu.org/copyleft/lgpl.html.
+# This file is part of cclib (http://cclib.github.io) and is distributed under
+# the terms of the BSD 3-Clause License.
 
 """Parser for NWChem output files"""
 
@@ -18,8 +13,8 @@ import re
 
 import numpy
 
-from . import logfileparser
-from . import utils
+from cclib.parser import logfileparser
+from cclib.parser import utils
 
 
 class NWChem(logfileparser.Logfile):
@@ -29,7 +24,7 @@ class NWChem(logfileparser.Logfile):
 
         # Call the __init__ method of the superclass
         super(NWChem, self).__init__(logname="NWChem", *args, **kwargs)
-        
+
     def __str__(self):
         """Return a string representation of the object."""
         return "NWChem log file %s" % (self.filename)
@@ -37,20 +32,9 @@ class NWChem(logfileparser.Logfile):
     def __repr__(self):
         """Return a representation of the object."""
         return 'NWChem("%s")' % (self.filename)
-    
+
     def normalisesym(self, label):
-        """Use standard symmetry labels instead of NWChem labels.
-
-        To normalise:
-        (1) If label is one of [SG, PI, PHI, DLTA], replace by [sigma, pi, phi, delta]
-        (2) replace any G or U by their lowercase equivalent
-
-        >>> sym = NWChem("dummyfile").normalisesym
-        >>> labels = ['A1', 'AG', 'A1G', "SG", "PI", "PHI", "DLTA", 'DLTU', 'SGG']
-        >>> map(sym, labels)
-        ['A1', 'Ag', 'A1g', 'sigma', 'pi', 'phi', 'delta', 'delta.u', 'sigma.g']
-        """
-        # FIXME if necessary
+        """NWChem does not require normalizing symmetry labels."""
         return label
 
     name2element = lambda self, lbl: "".join(itertools.takewhile(str.isalpha, str(lbl)))
@@ -58,6 +42,10 @@ class NWChem(logfileparser.Logfile):
     def extract(self, inputfile, line):
         """Extract information from the file object inputfile."""
 
+        #extract the version number first
+        if "Northwest Computational" in line:
+            self.metadata["package_version"] = line.split()[5]
+
         # This is printed in the input module, so should always be the first coordinates,
         # and contains some basic information we want to parse as well. However, this is not
         # the only place where the coordinates are printed during geometry optimization,
@@ -81,7 +69,7 @@ class NWChem(logfileparser.Logfile):
                     index, atomname, nuclear, x, y, z = line.split()
                 else:
                     index, atomname, tag, nuclear, x, y, z = line.split()
-                coords.append(list(map(float, [x,y,z])))
+                coords.append(list(map(float, [x, y, z])))
                 atomnos.append(int(float(nuclear)))
                 line = next(inputfile)
 
@@ -124,7 +112,7 @@ class NWChem(logfileparser.Logfile):
         #                      -----
         #  O (Oxygen)
         #  ----------
-        #            Exponent  Coefficients 
+        #            Exponent  Coefficients
         #       -------------- ---------------------------------------------------------
         #  1 S  1.30709320E+02  0.154329
         #  1 S  2.38088610E+01  0.535328
@@ -132,14 +120,14 @@ class NWChem(logfileparser.Logfile):
         #
         #  H1 (Hydrogen)
         #  -------------
-        #            Exponent  Coefficients 
+        #            Exponent  Coefficients
         #       -------------- ---------------------------------------------------------
         #  1 S  3.42525091E+00  0.154329
         # (...)
         #
         #  H2 (Hydrogen)
         #  -------------
-        #            Exponent  Coefficients 
+        #            Exponent  Coefficients
         #       -------------- ---------------------------------------------------------
         #  1 S  3.42525091E+00  0.154329
         # (...)
@@ -170,7 +158,7 @@ class NWChem(logfileparser.Logfile):
                             assert shell[0] == type
                         exp = float(exp)
                         coeff = float(coeff)
-                        shell[1].append((exp,coeff))
+                        shell[1].append((exp, coeff))
                         line = next(inputfile)
                     shells.append(shell)
                     line = next(inputfile)
@@ -178,7 +166,7 @@ class NWChem(logfileparser.Logfile):
 
             gbasis = []
             for i in range(self.natom):
-                atomtype = utils.PeriodicTable().element[self.atomnos[i]]
+                atomtype = self.table.element[self.atomnos[i]]
                 gbasis.append(gbasis_dict[atomtype])
 
             self.set_attribute('gbasis', gbasis)
@@ -190,33 +178,66 @@ class NWChem(logfileparser.Logfile):
         # listed in this summary of the AO basis. Similar to previous section, here
         # we assume all atoms of the same element have the same basis sets, but
         # this will probably need to be revised later.
-        if line.strip() == """Summary of "ao basis" -> "ao basis" (cartesian)""":
 
-            self.skip_lines(inputfile, ['d', 'headers', 'd'])
+        # The section we can glean info about aonmaes looks like:
+        #
+        # Summary of "ao basis" -> "ao basis" (cartesian)
+        # ------------------------------------------------------------------------------
+        #       Tag                 Description            Shells   Functions and Types
+        # ---------------- ------------------------------  ------  ---------------------
+        # C                           sto-3g                  3        5   2s1p
+        # H                           sto-3g                  1        1   1s
+        #
+        # However, we need to make sure not to match the following entry lines:
+        #
+        # *  Summary of "ao basis" -> "" (cartesian)
+        # *  Summary of allocated global arrays
+        #
+        # Unfortantely, "ao basis" isn't unique because it can be renamed to anything for
+        # later reference: http://www.nwchem-sw.org/index.php/Basis
+        # It also appears that we have to handle cartesian vs. spherical
 
-            atombasis_dict = {}
+        if line[1:11] == "Summary of":
+            match = re.match(' Summary of "([^\"]*)" -> "([^\"]*)" \((.+)\)', line)
+
+            if match and match.group(1) == match.group(2):
+
+                self.skip_lines(inputfile, ['d', 'title', 'd'])
+
+                self.shells = {}
+                self.shells["type"] = match.group(3)
+
+                atombasis_dict = {}
 
-            line = next(inputfile)
-            while line.strip():
-                atomname, desc, shells, funcs, types = line.split()
-                atomelement = self.name2element(atomname)
-                atombasis_dict[atomelement] = int(funcs)
                 line = next(inputfile)
+                while line.strip():
+                    atomname, desc, shells, funcs, types = line.split()
+                    atomelement = self.name2element(atomname)
+                    self.metadata["basis_set"] = desc
 
-            last = 0
-            atombasis = []
-            for i in range(self.natom):
-                atomelement = utils.PeriodicTable().element[self.atomnos[i]]
-                nfuncs = atombasis_dict[atomelement]
-                atombasis.append(list(range(last,last+nfuncs)))
-                last = atombasis[-1][-1] + 1
+                    self.shells[atomname] = types
+                    atombasis_dict[atomelement] = int(funcs)
+                    line = next(inputfile)
 
-            self.set_attribute('atombasis', atombasis)
+                last = 0
+                atombasis = []
+                for atom in self.atomnos:
+                    atomelement = self.table.element[atom]
+                    nfuncs = atombasis_dict[atomelement]
+                    atombasis.append(list(range(last, last+nfuncs)))
+                    last = atombasis[-1][-1] + 1
+
+                self.set_attribute('atombasis', atombasis)
 
         # This section contains general parameters for Hartree-Fock calculations,
         # which do not contain the 'General Information' section like most jobs.
         if line.strip() == "NWChem SCF Module":
-            self.skip_lines(inputfile, ['d', 'b', 'b', 'title', 'b', 'b', 'b'])
+            # If the calculation doesn't have a title specified, there
+            # aren't as many lines to skip here.
+            self.skip_lines(inputfile, ['d', 'b', 'b'])
+            line = next(inputfile)
+            if line.strip():
+                self.skip_lines(inputfile, ['b', 'b', 'b'])
             line = next(inputfile)
             while line.strip():
                 if line[2:8] == "charge":
@@ -257,11 +278,11 @@ class NWChem(logfileparser.Logfile):
 
                 # These will be present only in the DFT module.
                 if "Convergence on energy requested" in line:
-                    target_energy = float(line.split()[-1].replace('D', 'E'))
+                    target_energy = self.float(line.split()[-1])
                 if "Convergence on density requested" in line:
-                    target_density = float(line.split()[-1].replace('D', 'E'))
+                    target_density = self.float(line.split()[-1])
                 if "Convergence on gradient requested" in line:
-                    target_gradient = float(line.split()[-1].replace('D', 'E'))
+                    target_gradient = self.float(line.split()[-1])
 
                 line = next(inputfile)
 
@@ -271,6 +292,46 @@ class NWChem(logfileparser.Logfile):
                     self.scftargets = []
                 self.scftargets.append([target_energy, target_density, target_gradient])
 
+        #DFT functional information
+        if "XC Information" in line:
+            line = next(inputfile)
+            line = next(inputfile)
+            self.metadata["functional"] = line.split()[0]
+
+        # If the full overlap matrix is printed, it looks like this:
+        #
+        # global array: Temp Over[1:60,1:60],  handle: -996
+        #
+        #            1           2           3           4           5           6
+        #       ----------- ----------- ----------- ----------- ----------- -----------
+        #   1       1.00000     0.24836    -0.00000    -0.00000     0.00000     0.00000
+        #   2       0.24836     1.00000     0.00000    -0.00000     0.00000     0.00030
+        #   3      -0.00000     0.00000     1.00000     0.00000     0.00000    -0.00014
+        # ...
+        if "global array: Temp Over[" in line:
+
+            self.set_attribute('nbasis', int(line.split('[')[1].split(',')[0].split(':')[1]))
+            self.set_attribute('nmo', int(line.split(']')[0].split(',')[1].split(':')[1]))
+
+            aooverlaps = []
+            while len(aooverlaps) < self.nbasis:
+
+                self.skip_line(inputfile, 'blank')
+
+                indices = [int(i) for i in inputfile.next().split()]
+                assert indices[0] == len(aooverlaps) + 1
+
+                self.skip_line(inputfile, "dashes")
+                data = [inputfile.next().split() for i in range(self.nbasis)]
+                indices = [int(d[0]) for d in data]
+                assert indices == list(range(1, self.nbasis+1))
+
+                for i in range(1, len(data[0])):
+                    vector = [float(d[i]) for d in data]
+                    aooverlaps.append(vector)
+
+            self.set_attribute('aooverlaps', aooverlaps)
+
         if line.strip() in ("The SCF is already converged", "The DFT is already converged"):
             if self.linesearch:
                 return
@@ -305,15 +366,24 @@ class NWChem(logfileparser.Logfile):
                     self.skip_line(inputfile, 'dashes')
                     line = next(inputfile)
                     while line.strip():
-                        iter,energy,gnorm,gmax,time = line.split()
-                        gnorm = float(gnorm.replace('D','E'))
+                        it, energy, gnorm, gmax, time = line.split()
+                        gnorm = self.float(gnorm)
                         values.append([gnorm])
-                        line = next(inputfile)
+                        try:
+                            line = next(inputfile)
+                        # Is this the end of the file for some reason?
+                        except StopIteration:
+                            self.logger.warning('File terminated before end of last SCF! Last gradient norm: {}'.format(gnorm))
+                            break
                     if not hasattr(self, 'scfvalues'):
                         self.scfvalues = []
                     self.scfvalues.append(values)
 
-                line = next(inputfile)
+                try:
+                    line = next(inputfile)
+                except StopIteration:
+                    self.logger.warning('File terminated?')
+                    break
 
         # The SCF for DFT does not use the same algorithm as Hartree-Fock, but always
         # seems to use the following format to report SCF convergence:
@@ -347,12 +417,17 @@ class NWChem(logfileparser.Logfile):
                 # ...
                 if len(line[17:].split()) == 6:
                     iter, energy, deltaE, dens, diis, time = line[17:].split()
-                    val_energy = float(deltaE.replace('D', 'E'))
-                    val_density = float(dens.replace('D', 'E'))
-                    val_gradient = float(diis.replace('D', 'E'))
+                    val_energy = self.float(deltaE)
+                    val_density = self.float(dens)
+                    val_gradient = self.float(diis)
                     values.append([val_energy, val_density, val_gradient])
 
-                line = next(inputfile)
+                try:
+                    line = next(inputfile)
+                # Is this the end of the file for some reason?
+                except StopIteration:
+                    self.logger.warning('File terminated before end of last SCF! Last error: {}'.format(diis))
+                    break
 
             if not hasattr(self, 'scfvalues'):
                 self.scfvalues = []
@@ -372,19 +447,19 @@ class NWChem(logfileparser.Logfile):
         #@ Step       Energy      Delta E   Gmax     Grms     Xrms     Xmax   Walltime
         #@ ---- ---------------- -------- -------- -------- -------- -------- --------
         #@    0    -379.76896249  0.0D+00  0.04567  0.01110  0.00000  0.00000      4.2
-        #                                                       ok       ok  
+        #                                                       ok       ok
         #
         # However, for subsequent step the format is a bit different:
         #
         #  Step       Energy      Delta E   Gmax     Grms     Xrms     Xmax   Walltime
         #  ---- ---------------- -------- -------- -------- -------- -------- --------
         #@    2    -379.77794602 -7.4D-05  0.00118  0.00023  0.00440  0.01818     14.8
-        #                                              ok     
+        #                                              ok
         #
         # There is also a summary of the line search (which we don't use now), like this:
         #
-        # Line search: 
-        #     step= 1.00 grad=-1.8D-05 hess= 8.9D-06 energy=   -379.777955 mode=accept  
+        # Line search:
+        #     step= 1.00 grad=-1.8D-05 hess= 8.9D-06 energy=   -379.777955 mode=accept
         # new step= 1.00                   predicted energy=   -379.777955
         #
         if line[10:14] == "Step":
@@ -435,6 +510,12 @@ class NWChem(logfileparser.Logfile):
             if not hasattr(self, 'optdone'):
                 self.optdone = []
 
+        # extract the theoretical method
+        if "Total SCF energy" in line:
+            self.metadata["methods"].append("HF")
+        if "Total DFT energy" in line:
+            self.metadata["methods"].append("DFT")
+
         # The line containing the final SCF energy seems to be always identifiable like this.
         if "Total SCF energy" in line or "Total DFT energy" in line:
 
@@ -458,14 +539,14 @@ class NWChem(logfileparser.Logfile):
         #       Symmetry analysis of molecular orbitals - final
         #       -----------------------------------------------
         #
-        #  Numbering of irreducible representations: 
+        #  Numbering of irreducible representations:
         #
-        #     1 ag          2 au          3 bg          4 bu      
+        #     1 ag          2 au          3 bg          4 bu
         #
         #  Orbital symmetries:
         #
-        #     1 bu          2 ag          3 bu          4 ag          5 bu      
-        #     6 ag          7 bu          8 ag          9 bu         10 ag 
+        #     1 bu          2 ag          3 bu          4 ag          5 bu
+        #     6 ag          7 bu          8 ag          9 bu         10 ag
         # ...
         if line.strip() == "Symmetry analysis of molecular orbitals - final":
 
@@ -500,49 +581,51 @@ class NWChem(logfileparser.Logfile):
         #
         # Vector    1  Occ=2.000000D+00  E=-1.104059D+01  Symmetry=bu
         #              MO Center=  1.4D-17,  0.0D+00, -6.5D-37, r^2= 2.1D+00
-        #   Bfn.  Coefficient  Atom+Function         Bfn.  Coefficient  Atom+Function  
+        #   Bfn.  Coefficient  Atom+Function         Bfn.  Coefficient  Atom+Function
         #  ----- ------------  ---------------      ----- ------------  ---------------
-        #     1      0.701483   1 C  s                 6     -0.701483   2 C  s         
+        #     1      0.701483   1 C  s                 6     -0.701483   2 C  s
         #
         # Vector    2  Occ=2.000000D+00  E=-1.104052D+01  Symmetry=ag
         # ...
         # Vector   12  Occ=2.000000D+00  E=-1.020253D+00  Symmetry=bu
         #              MO Center= -1.4D-17, -5.6D-17,  2.9D-34, r^2= 7.9D+00
-        #   Bfn.  Coefficient  Atom+Function         Bfn.  Coefficient  Atom+Function  
+        #   Bfn.  Coefficient  Atom+Function         Bfn.  Coefficient  Atom+Function
         #  ----- ------------  ---------------      ----- ------------  ---------------
-        #    36     -0.298699  11 C  s                41      0.298699  12 C  s         
-        #     2      0.270804   1 C  s                 7     -0.270804   2 C  s         
+        #    36     -0.298699  11 C  s                41      0.298699  12 C  s
+        #     2      0.270804   1 C  s                 7     -0.270804   2 C  s
         #    48     -0.213655  15 C  s                53      0.213655  16 C  s
         # ...
         #
         if "Final" in line and "Molecular Orbital Analysis" in line:
 
             # Unrestricted jobs have two such blocks, for alpha and beta orbitals, and
-            # we need to keep track of which one we're parsing (always alpha in restricted case). 
+            # we need to keep track of which one we're parsing (always alpha in restricted case).
             unrestricted = ("Alpha" in line) or ("Beta" in line)
             alphabeta = int("Beta" in line)
 
             self.skip_lines(inputfile, ['dashes', 'blank'])
 
+            nvectors = []
+            mooccnos = []
             energies = []
             symmetries = [None]*self.nbasis
             line = next(inputfile)
-            homo = 0
             while line[:7] == " Vector":
 
                 # Note: the vector count starts from 1 in NWChem.
                 nvector = int(line[7:12])
+                nvectors.append(nvector)
 
                 # A nonzero occupancy for SCF jobs means the orbital is occupied.
-                if ("Occ=2.0" in line) or ("Occ=1.0" in line):
-                    homo = nvector-1
+                mooccno = int(self.float(line[18:30]))
+                mooccnos.append(mooccno)
 
                 # If the printout does not start from the first MO, assume None for all previous orbitals.
                 if len(energies) == 0 and nvector > 1:
-                    for i in range(1,nvector):
+                    for i in range(1, nvector):
                         energies.append(None)
 
-                energy = float(line[34:47].replace('D','E'))
+                energy = self.float(line[34:47])
                 energy = utils.convertor(energy, "hartree", "eV")
                 energies.append(energy)
 
@@ -575,7 +658,20 @@ class NWChem(logfileparser.Logfile):
 
             if not hasattr(self, 'homos') or (len(self.homos) > alphabeta):
                 self.homos = []
-            self.homos.append(homo)
+            nvector_index = mooccnos.index(0) - 1
+            if nvector_index > -1:
+                self.homos.append(nvectors[nvector_index] - 1)
+            else:
+                self.homos.append(-1)
+            # If this was a restricted open-shell calculation, append
+            # to HOMOs twice since only one Molecular Orbital Analysis
+            # section is in the output file.
+            if (not unrestricted) and (1 in mooccnos):
+                nvector_index = mooccnos.index(1) - 1
+                if nvector_index > -1:
+                    self.homos.append(nvectors[nvector_index] - 1)
+                else:
+                    self.homos.append(-1)
 
         # This is where the full MO vectors are printed, but a special directive is needed for it:
         #
@@ -583,9 +679,9 @@ class NWChem(logfileparser.Logfile):
         #                                 ----------------
         #
         #
-        # global array: alpha evecs[1:60,1:60],  handle: -995 
+        # global array: alpha evecs[1:60,1:60],  handle: -995
         #
-        #            1           2           3           4           5           6  
+        #            1           2           3           4           5           6
         #       ----------- ----------- ----------- ----------- ----------- -----------
         #   1      -0.69930    -0.69930    -0.02746    -0.02769    -0.00313    -0.02871
         #   2      -0.03156    -0.03135     0.00410     0.00406     0.00078     0.00816
@@ -611,22 +707,22 @@ class NWChem(logfileparser.Logfile):
                 nmo = int(size.split(',')[1].split(':')[1])
                 self.set_attribute('nbasis', nbasis)
                 self.set_attribute('nmo', nmo)
-            
+
                 self.skip_line(inputfile, 'blank')
                 mocoeffs = []
                 while len(mocoeffs) < self.nmo:
-                    nmos = list(map(int,next(inputfile).split()))
-                    assert len(mocoeffs) == nmos[0]-1
+                    nmos = list(map(int, next(inputfile).split()))
+                    assert len(mocoeffs) == nmos[0] - 1
                     for n in nmos:
                         mocoeffs.append([])
                     self.skip_line(inputfile, 'dashes')
-                    for nb in range(nbasis):                
+                    for nb in range(nbasis):
                         line = next(inputfile)
                         index = int(line.split()[0])
                         assert index == nb+1
-                        coefficients = list(map(float,line.split()[1:]))
+                        coefficients = list(map(float, line.split()[1:]))
                         assert len(coefficients) == len(nmos)
-                        for i,c in enumerate(coefficients):
+                        for i, c in enumerate(coefficients):
                             mocoeffs[nmos[i]-1].append(c)
                     self.skip_line(inputfile, 'blank')
                 self.mocoeffs.append(mocoeffs)
@@ -659,7 +755,9 @@ class NWChem(logfileparser.Logfile):
                 line = next(inputfile)
             self.atomcharges['mulliken'] = charges
 
-        # If the full overlap matrix is printed, it looks like this:
+        # Not the the 'overlap population' as printed in the Mulliken population analysis,
+        # is not the same thing as the 'overlap matrix'. In fact, it is the overlap matrix
+        # multiplied elementwise times the density matrix.
         #
         #          ----------------------------
         #          Mulliken population analysis
@@ -673,7 +771,7 @@ class NWChem(logfileparser.Logfile):
         #    2   1 C  s           -0.0535883400   0.8281341291   0.0000000000  -0.0000000000   0.0000000000   0.0000039991  -0.0009906747
         # ...
         #
-        # Also, DFT does not seem to print the separate listing of Mulliken charges
+        # DFT does not seem to print the separate listing of Mulliken charges
         # by default, but they are printed by this modules later on. They are also print
         # for Hartree-Fock runs, though, so in that case make sure they are consistent.
         if line.strip() == "Mulliken population analysis":
@@ -681,7 +779,7 @@ class NWChem(logfileparser.Logfile):
             self.skip_lines(inputfile, ['d', 'b', 'total_overlap_population', 'b'])
 
             overlaps = []
-            line= next(inputfile)
+            line = next(inputfile)
             while all([c.isdigit() for c in line.split()]):
 
                 # There is always a line with the MO indices printed in thie block.
@@ -698,14 +796,12 @@ class NWChem(logfileparser.Logfile):
                 # Now we can iterate or atomic orbitals.
                 for nao in range(self.nbasis):
                     data = list(map(float, line.split()[4:]))
-                    for i,d in enumerate(data):
+                    for i, d in enumerate(data):
                         overlaps[indices[i]].append(d)
                     line = next(inputfile)
 
                 line = next(inputfile)
 
-            self.aooverlaps = overlaps
-
             # This header should be printed later, before the charges are print, which of course
             # are just sums of the overlaps and could be calculated. But we just go ahead and
             # parse them, make sure they're consistent with previously parsed values and
@@ -735,7 +831,7 @@ class NWChem(logfileparser.Logfile):
         # to the values in Debye, which are also printed. But we can also just convert them
         # right away and so parse a little bit less. Note how the reference point is print
         # here within the block nicely, as it is for all moment later.
-        # 
+        #
         #          -------------
         #          Dipole Moment
         #          -------------
@@ -791,8 +887,8 @@ class NWChem(logfileparser.Logfile):
         # Center of charge (in au) is the expansion point
         #         X =       0.0000000 Y =       0.0000000 Z =       0.0000000
         #
-        # < R**2 > = ********** a.u.  ( 1 a.u. = 0.280023 10**(-16) cm**2 ) 
-        # ( also called diamagnetic susceptibility ) 
+        # < R**2 > = ********** a.u.  ( 1 a.u. = 0.280023 10**(-16) cm**2 )
+        # ( also called diamagnetic susceptibility )
         #
         #   Second moments in atomic units
         #
@@ -835,7 +931,7 @@ class NWChem(logfileparser.Logfile):
             # and octupole output. So, it is perfectly possible to print the quadrupole
             # and not the dipole... if that is the case set the former to None and
             # issue a warning. Also, a regression has been added to cover this case.
-            if not hasattr(self, 'moments') or len(self.moments) <2:
+            if not hasattr(self, 'moments') or len(self.moments) < 2:
                 self.logger.warning("Found quadrupole moments but no previous dipole")
                 self.moments = [self.reference, None, quadrupole]
             else:
@@ -904,20 +1000,105 @@ class NWChem(logfileparser.Logfile):
                     assert self.moments[3] == octupole
 
         if "Total MP2 energy" in line:
+            self.metadata["methods"].append("MP2")
             mpenerg = float(line.split()[-1])
             if not hasattr(self, "mpenergies"):
                 self.mpenergies = []
             self.mpenergies.append([])
             self.mpenergies[-1].append(utils.convertor(mpenerg, "hartree", "eV"))
 
-        if "CCSD(T) total energy / hartree" in line:
+        if "CCSD total energy / hartree" in line or "total CCSD energy:" in line:
+            self.metadata["methods"].append("CCSD")
             ccenerg = float(line.split()[-1])
             if not hasattr(self, "ccenergies"):
                 self.ccenergies = []
             self.ccenergies.append([])
             self.ccenergies[-1].append(utils.convertor(ccenerg, "hartree", "eV"))
 
+        if "CCSD(T) total energy / hartree" in line:
+            self.metadata["methods"].append("CCSD(T)")
+            ccenerg = float(line.split()[-1])
+            if not hasattr(self, "ccenergies"):
+                self.ccenergies = []
+            self.ccenergies.append([])
+            self.ccenergies[-1].append(utils.convertor(ccenerg, "hartree", "eV"))
 
-if __name__ == "__main__":
-    import doctest, nwchemparser
-    doctest.testmod(nwchemparser, verbose=False)
+        # Static and dynamic polarizability.
+        if "Linear Response polarizability / au" in line:
+            if not hasattr(self, "polarizabilities"):
+                self.polarizabilities = []
+            polarizability = []
+            line = next(inputfile)
+            assert line.split()[0] == "Frequency"
+            line = next(inputfile)
+            assert line.split()[0] == "Wavelength"
+            self.skip_lines(inputfile, ['coordinates', 'd'])
+            for _ in range(3):
+                line = next(inputfile)
+                polarizability.append(line.split()[1:])
+            self.polarizabilities.append(numpy.array(polarizability))
+
+    def after_parsing(self):
+        """NWChem-specific routines for after parsing file.
+
+        Currently, expands self.shells() into self.aonames.
+        """
+
+        # setup a few necessary things, including a regular expression
+        # for matching the shells
+        table = utils.PeriodicTable()
+        elements = [table.element[x] for x in self.atomnos]
+        pattern = re.compile("(\ds)+(\dp)*(\dd)*(\df)*(\dg)*")
+
+        labels = {}
+        labels['s'] = ["%iS"]
+        labels['p'] = ["%iPX", "%iPY", "%iPZ"]
+        if self.shells['type'] == 'spherical':
+            labels['d'] = ['%iD-2', '%iD-1', '%iD0', '%iD1', '%iD2']
+            labels['f'] = ['%iF-3', '%iF-2', '%iF-1', '%iF0',
+                           '%iF1', '%iF2', '%iF3']
+            labels['g'] = ['%iG-4', '%iG-3', '%iG-2', '%iG-1', '%iG0',
+                           '%iG1', '%iG2', '%iG3', '%iG4']
+        elif self.shells['type'] == 'cartesian':
+            labels['d'] = ['%iDXX', '%iDXY', '%iDXZ',
+                           '%iDYY', '%iDYZ',
+                           '%iDZZ']
+            labels['f'] = ['%iFXXX', '%iFXXY', '%iFXXZ',
+                           '%iFXYY', '%iFXYZ', '%iFXZZ',
+                           '%iFYYY', '%iFYYZ', '%iFYZZ',
+                           '%iFZZZ']
+            labels['g'] = ['%iGXXXX', '%iGXXXY', '%iGXXXZ',
+                           '%iGXXYY', '%iGXXYZ', '%iGXXZZ',
+                           '%iGXYYY', '%iGXYYZ', '%iGXYZZ',
+                           '%iGXZZZ', '%iGYYYY', '%iGYYYZ',
+                           '%iGYYZZ', '%iGYZZZ', '%iGZZZZ']
+        else:
+            self.logger.warning("Found a non-standard aoname representation type.")
+            return
+
+        # now actually build aonames
+        # involves expanding 2s1p into appropriate types
+
+        self.aonames = []
+        for i, element in enumerate(elements):
+            try:
+                shell_text = self.shells[element]
+            except KeyError:
+                del self.aonames
+                msg = "Cannot determine aonames for at least one atom."
+                self.logger.warning(msg)
+                break
+
+            prefix = "%s%i_" % (element, i + 1)  # (e.g. C1_)
+
+            matches = pattern.match(shell_text)
+            for j, group in enumerate(matches.groups()):
+                if group is None:
+                    continue
+
+                count = int(group[:-1])
+                label = group[-1]
+
+                for k in range(count):
+                    temp = [x % (j + k + 1) for x in labels[label]]
+                    self.aonames.extend([prefix + x for x in temp])
